@@ -71,7 +71,7 @@ import java.util.StringTokenizer;
  * Default implementation of the site tool.
  *
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: DefaultSiteTool.java 746295 2009-02-20 17:08:56Z dennisl $
+ * @version $Id: DefaultSiteTool.java 884306 2009-11-25 22:27:01Z hboutemy $
  *
  * @plexus.component role="org.apache.maven.doxia.tools.SiteTool" role-hint="default"
  */
@@ -211,7 +211,7 @@ public class DefaultSiteTool
         {
             try
             {
-                toUrl = new File( getNormalizedPath( to ) ).toURL();
+                toUrl = new File( getNormalizedPath( to ) ).toURI().toURL();
             }
             catch ( MalformedURLException e1 )
             {
@@ -227,7 +227,7 @@ public class DefaultSiteTool
         {
             try
             {
-                fromUrl = new File( getNormalizedPath( from ) ).toURL();
+                fromUrl = new File( getNormalizedPath( from ) ).toURI().toURL();
             }
             catch ( MalformedURLException e1 )
             {
@@ -424,7 +424,7 @@ public class DefaultSiteTool
 
     /** {@inheritDoc} */
     public File getSiteDescriptorFromRepository( MavenProject project, ArtifactRepository localRepository,
-                                                 List remoteArtifactRepositories, Locale locale )
+                                                 List repositories, Locale locale )
         throws SiteToolException
     {
         if ( project == null )
@@ -435,7 +435,7 @@ public class DefaultSiteTool
         {
             throw new IllegalArgumentException( "The parameter 'localRepository' can not be null" );
         }
-        if ( remoteArtifactRepositories == null )
+        if ( repositories == null )
         {
             throw new IllegalArgumentException( "The parameter 'remoteArtifactRepositories' can not be null" );
         }
@@ -447,7 +447,7 @@ public class DefaultSiteTool
 
         try
         {
-            return resolveSiteDescriptor( project, localRepository, remoteArtifactRepositories, locale );
+            return resolveSiteDescriptor( project, localRepository, repositories, locale );
         }
         catch ( ArtifactNotFoundException e )
         {
@@ -619,7 +619,7 @@ public class DefaultSiteTool
     }
 
     /** {@inheritDoc} */
-    public String getInterpolatedSiteDescriptorContent( Map props, MavenProject project, String siteDescriptorContent,
+    public String getInterpolatedSiteDescriptorContent( Map props, MavenProject aProject, String siteDescriptorContent,
                                                         String inputEncoding, String outputEncoding )
         throws SiteToolException
     {
@@ -627,7 +627,7 @@ public class DefaultSiteTool
         {
             throw new IllegalArgumentException( "The parameter 'props' can not be null" );
         }
-        if ( project == null )
+        if ( aProject == null )
         {
             throw new IllegalArgumentException( "The parameter 'project' can not be null" );
         }
@@ -668,9 +668,9 @@ public class DefaultSiteTool
                                          e );
         }
 
-        interpolator.addValueSource( new ObjectBasedValueSource( project ) );
+        interpolator.addValueSource( new ObjectBasedValueSource( aProject ) );
 
-        interpolator.addValueSource( new MapBasedValueSource( project.getProperties() ) );
+        interpolator.addValueSource( new MapBasedValueSource( aProject.getProperties() ) );
 
         siteDescriptorContent = interpolator.interpolate( siteDescriptorContent, "project" );
 
@@ -688,10 +688,10 @@ public class DefaultSiteTool
     }
 
     /** {@inheritDoc} */
-    public MavenProject getParentProject( MavenProject project, List reactorProjects,
+    public MavenProject getParentProject( MavenProject aProject, List reactorProjects,
                                           ArtifactRepository localRepository )
     {
-        if ( project == null )
+        if ( aProject == null )
         {
             throw new IllegalArgumentException( "The parameter 'project' can not be null" );
         }
@@ -706,7 +706,7 @@ public class DefaultSiteTool
 
         MavenProject parentProject = null;
 
-        MavenProject origParent = project.getParent();
+        MavenProject origParent = aProject.getParent();
         if ( origParent != null )
         {
             Iterator reactorItr = reactorProjects.iterator();
@@ -724,11 +724,11 @@ public class DefaultSiteTool
                 }
             }
 
-            if ( parentProject == null && project.getBasedir() != null )
+            if ( parentProject == null && aProject.getBasedir() != null )
             {
                 try
                 {
-                    File pomFile = new File( project.getBasedir(), project.getModel().getParent().getRelativePath() );
+                    File pomFile = new File( aProject.getBasedir(), aProject.getModel().getParent().getRelativePath() );
 
                     if ( pomFile.isDirectory() )
                     {
@@ -736,13 +736,16 @@ public class DefaultSiteTool
                     }
                     pomFile = new File( getNormalizedPath( pomFile.getPath() ) );
 
-                    MavenProject mavenProject = mavenProjectBuilder.build( pomFile, localRepository, null );
-
-                    if ( mavenProject.getGroupId().equals( origParent.getGroupId() )
-                        && mavenProject.getArtifactId().equals( origParent.getArtifactId() )
-                        && mavenProject.getVersion().equals( origParent.getVersion() ) )
+                    if ( pomFile.isFile() )
                     {
-                        parentProject = mavenProject;
+                        MavenProject mavenProject = mavenProjectBuilder.build( pomFile, localRepository, null );
+
+                        if ( mavenProject.getGroupId().equals( origParent.getGroupId() )
+                            && mavenProject.getArtifactId().equals( origParent.getArtifactId() )
+                            && mavenProject.getVersion().equals( origParent.getVersion() ) )
+                        {
+                            parentProject = mavenProject;
+                        }
                     }
                 }
                 catch ( ProjectBuildingException e )
@@ -755,7 +758,7 @@ public class DefaultSiteTool
             {
                 try
                 {
-                    parentProject = mavenProjectBuilder.buildFromRepository( project.getParentArtifact(), project
+                    parentProject = mavenProjectBuilder.buildFromRepository( aProject.getParentArtifact(), aProject
                         .getRemoteArtifactRepositories(), localRepository );
                     getLogger().info( "Parent project loaded from repository." );
                 }
@@ -820,27 +823,31 @@ public class DefaultSiteTool
                 }
 
                 parentUrl = getRelativePath( parentUrl, project.getUrl() );
-
-                if ( menu.getName() == null )
-                {
-                    menu.setName( i18n.getString( "site-tool", locale, "decorationModel.menu.parentproject" ) );
-                }
-
-                MenuItem item = new MenuItem();
-                item.setName( parentProject.getName() );
-                item.setHref( parentUrl );
-                menu.addItem( item );
             }
             else
             {
-                decorationModel.removeMenuRef( "parent" );
+                // parent has no url, assume relative path is given by site structure
+                String parentPath = parentProject.getBasedir().getAbsolutePath();
+                String projectPath = project.getBasedir().getAbsolutePath();
+                parentUrl = getRelativePath( parentPath, projectPath ) + "/index.html";
             }
+
+            if ( menu.getName() == null )
+            {
+                menu.setName( i18n.getString( "site-tool", locale, "decorationModel.menu.parentproject" ) );
+            }
+
+            MenuItem item = new MenuItem();
+            item.setName( parentProject.getName() );
+            item.setHref( parentUrl );
+            menu.addItem( item );
         }
     }
 
     /**
      * {@inheritDoc}
-     * @deprecated Please use {@link #populateParentMenu(DecorationModel, Locale, MavenProject, MavenProject, boolean)} instead
+     * @deprecated Please use
+     *      {@link #populateParentMenu(DecorationModel, Locale, MavenProject, MavenProject, boolean)} instead
      */
     public void populateProjectParentMenu( DecorationModel decorationModel, Locale locale, MavenProject project,
                                            MavenProject parentProject, boolean keepInheritedRefs )
@@ -850,7 +857,9 @@ public class DefaultSiteTool
 
     /**
      * {@inheritDoc}
-     * @deprecated Please use {@link #populateModulesMenu(MavenProject, List, ArtifactRepository, DecorationModel, Locale, boolean)} instead
+     * @deprecated Please use
+     *      {@link #populateModulesMenu(MavenProject, List, ArtifactRepository, DecorationModel, Locale, boolean)}
+     *      instead
      */
     public void populateModules( MavenProject project, List reactorProjects, ArtifactRepository localRepository,
                                  DecorationModel decorationModel, Locale locale, boolean keepInheritedRefs )
@@ -1204,6 +1213,7 @@ public class DefaultSiteTool
         }
 
         String siteDescriptorContent = null;
+        long siteDescriptorLastModified = 0L;
         try
         {
             if ( siteDescriptor != null && siteDescriptor.exists() )
@@ -1211,6 +1221,7 @@ public class DefaultSiteTool
                 getLogger().debug( "Reading site descriptor from " + siteDescriptor );
                 Reader siteDescriptorReader = ReaderFactory.newXmlReader( siteDescriptor );
                 siteDescriptorContent = IOUtil.toString( siteDescriptorReader );
+                siteDescriptorLastModified = siteDescriptor.lastModified();
             }
         }
         catch ( IOException e )
@@ -1225,6 +1236,7 @@ public class DefaultSiteTool
                                                                           inputEncoding, outputEncoding );
 
             decoration = readDecorationModel( siteDescriptorContent );
+            decoration.setLastModified( siteDescriptorLastModified );
         }
 
         MavenProject parentProject = getParentProject( project, reactorProjects, localRepository );
@@ -1236,15 +1248,14 @@ public class DefaultSiteTool
 
             if ( decoration == null )
             {
-                decoration = parent;
-            }
-            else
-            {
-                assembler.assembleModelInheritance( project.getName(), decoration, parent, project.getUrl(),
-                                                    parentProject.getUrl() == null ? project.getUrl() : parentProject
-                                                        .getUrl() );
+                // we have no site descriptor: merge the parent into an empty one
+                decoration = new DecorationModel();
             }
+
+            assembler.assembleModelInheritance( project.getName(), decoration, parent, project.getUrl(),
+                        parentProject.getUrl() == null ? project.getUrl() : parentProject.getUrl() );
         }
+
         if ( decoration != null && decoration.getSkin() != null )
         {
             getLogger().debug( "Skin used: " + decoration.getSkin() );
